@@ -36,99 +36,94 @@
 #include "hash_table.h"
 
 static int units_initialized = 0;
-static struct hash_table *conversion_fields = NULL;
-static struct hash_table *multiplier_of     = NULL;
+struct hash_table *conversion_fields = NULL;
 
 struct conversion_field {
 	char *name;
-	char *internal_unit;
-	char *external_unit;
+	char  *internal_unit;
+	char  *external_unit;
+	char  *base_unit;
+	double external_to_internal; // internal = external * this factor.
+	double internal_to_base;     // base     = internal * this factor.
 	int  float_flag;
 };
 
-void add_conversion_field(const char *name, const char *internal, const char *external, int float_flag) {
-	struct conversion_field *c = malloc(sizeof(struct conversion_field));
-
-	c->name          = xxstrdup(name);
-	c->internal_unit = xxstrdup(internal);
-	c->external_unit = xxstrdup(external);
-	c->float_flag    = float_flag;
-
-	hash_table_insert(conversion_fields, name, (void *) c);
-}
+struct multiplier {
+	double val;
+};
 
 void initialize_units() {
 	units_initialized = 1;
 
 	conversion_fields = hash_table_create(32, 0);
-	multiplier_of     = hash_table_create(32, 0);
 
-	add_conversion_field("wall_time",                "us",     "s",       1);
-	add_conversion_field("cpu_time",                 "us",     "s",       1);
-	add_conversion_field("start",                    "us",     "us",      0);
-	add_conversion_field("end",                      "us",     "us",      0);
-	add_conversion_field("memory",                   "MB",     "MB",      0);
-	add_conversion_field("virtual_memory",           "MB",     "MB",      0);
-	add_conversion_field("swap_memory",              "MB",     "MB",      0);
-	add_conversion_field("disk",                     "MB",     "MB",      0);
-	add_conversion_field("bytes_read",               "B",      "MB",      1);
-	add_conversion_field("bytes_written",            "B",      "MB",      1);
-	add_conversion_field("bytes_received",           "B",      "MB",      1);
-	add_conversion_field("bytes_sent",               "B",      "MB",      1);
-	add_conversion_field("bandwidth",                "bps",    "Mbps",    1);
-	add_conversion_field("cores",                    "cores",  "cores",   0);
-	add_conversion_field("max_concurrent_processes", "procs",  "procs",   0);
-	add_conversion_field("total_processes",          "procs",  "procs",   0);
-	add_conversion_field("total_files",              "files",  "files",   0);
-
-	/* to internal units */
-	hash_table_insert(multiplier_of, "us",    (void **) 1);
-	hash_table_insert(multiplier_of, "B",     (void **) 1);
-	hash_table_insert(multiplier_of, "bps",   (void **) 1);
-	hash_table_insert(multiplier_of, "cores", (void **) 1);
-	hash_table_insert(multiplier_of, "procs", (void **) 1);
-	hash_table_insert(multiplier_of, "files", (void **) 1);
-
-	/* size units */
-	hash_table_insert(multiplier_of, "kB", (void **) KILOBYTE);
-	hash_table_insert(multiplier_of, "MB", (void **) MEGABYTE);
-	hash_table_insert(multiplier_of, "GB", (void **) GIGABYTE);
-
-	/* time units */
-	hash_table_insert(multiplier_of, "s",  (void **) USECOND);
-	hash_table_insert(multiplier_of, "ms", (void **) 1000);
-
-	/* rate units */
-	hash_table_insert(multiplier_of, "Gbps",  (void **) 1000000000);
-	hash_table_insert(multiplier_of, "Mbps",  (void **) 1000000);
+                                   // name                 internal  external      base       exttoint     inttobase    float_flag
+	rmsummary_add_conversion_field("wall_time",                "us",      "s",      "s",       USECOND,  1.0/USECOND,   1);
+	rmsummary_add_conversion_field("cpu_time",                 "us",      "s",      "s",       USECOND,  1.0/USECOND,   1);
+	rmsummary_add_conversion_field("start",                    "us",      "us",     "s",       1,        1.0/USECOND,   0);
+	rmsummary_add_conversion_field("end",                      "us",      "us",     "s",       1,        1.0/USECOND,   0);
+	rmsummary_add_conversion_field("memory",                   "MB",      "MB",     "B",       1,        MEGABYTE,      0);
+	rmsummary_add_conversion_field("virtual_memory",           "MB",      "MB",     "B",       1,        MEGABYTE,      0);
+	rmsummary_add_conversion_field("swap_memory",              "MB",      "MB",     "B",       1,        MEGABYTE,      0);
+	rmsummary_add_conversion_field("disk",                     "MB",      "MB",     "B",       1,        MEGABYTE,      0);
+	rmsummary_add_conversion_field("bytes_read",               "B",       "MB",     "B",       MEGABYTE, 1,             1);
+	rmsummary_add_conversion_field("bytes_written",            "B",       "MB",     "B",       MEGABYTE, 1,             1);
+	rmsummary_add_conversion_field("bytes_received",           "B",       "MB",     "B",       MEGABYTE, 1,             1);
+	rmsummary_add_conversion_field("bytes_sent",               "B",       "MB",     "B",       MEGABYTE, 1,             1);
+	rmsummary_add_conversion_field("bandwidth",                "bps",     "Mbps",   "bps",     1000,     1,             1);
+	rmsummary_add_conversion_field("cores",                    "cores",   "cores",  "cores",   1,        1,             0);
+	rmsummary_add_conversion_field("cores_avg",                "mcores",  "cores",  "cores",   1000,     1/1000.0,      1);
+	rmsummary_add_conversion_field("machine_cpus",             "cores",   "cores",  "cores",   1,        1,             0);
+	rmsummary_add_conversion_field("machine_load",             "mprocs",  "procs",  "procs",   1000,     1/1000.0,      1);
+	rmsummary_add_conversion_field("max_concurrent_processes", "procs",   "procs",  "procs",   1,        1,             0);
+	rmsummary_add_conversion_field("total_processes",          "procs",   "procs",  "procs",   1,        1,             0);
+	rmsummary_add_conversion_field("total_files",              "files",   "files",  "files",   1,        1,             0);
 }
 
-int rmsummary_to_internal_unit(const char *field, double input_number, int64_t *output_number, const char *unit) {
+void rmsummary_add_conversion_field(const char *name, const char *internal, const char *external, const char *base, double exttoint, double inttobase, int float_flag) {
 
 	if(!units_initialized)
 		initialize_units();
 
-	uintptr_t multiplier = (uintptr_t) hash_table_lookup(multiplier_of, unit);
-
-	if(!multiplier) {
-		warn(D_DEBUG, "Unknown units: '%s'", unit);
-		return 0;
+	struct conversion_field *c = hash_table_lookup(conversion_fields, name);
+	if(c) {
+		free(c->name);
+		free(c->internal_unit);
+		free(c->external_unit);
+		free(c->base_unit);
+	}
+	else {
+		c = malloc(sizeof(struct conversion_field));
 	}
 
-	input_number *= multiplier;
+	c->name                 = xxstrdup(name);
+	c->internal_unit        = xxstrdup(internal);
+	c->external_unit        = xxstrdup(external);
+	c->base_unit            = xxstrdup(base);
+	c->external_to_internal = exttoint;
+	c->internal_to_base     = inttobase;
+	c->float_flag           = float_flag;
+
+	hash_table_insert(conversion_fields, name, (void *) c);
+}
+
+int rmsummary_to_internal_unit(const char *field, double input_number, int64_t *output_number, const char *external_unit) {
+	if(!units_initialized)
+		initialize_units();
+
+	double factor = 1;
 
 	struct conversion_field *cf = hash_table_lookup(conversion_fields, field);
-	if(!cf) {
-		warn(D_DEBUG, "Unknown field: '%s'", field);
-		return 0;
+
+	if(!cf || strcmp(cf->internal_unit, external_unit) == 0) {
+		factor = 1;
+	} else if(strcmp(cf->external_unit, external_unit) == 0 || strcmp("external", external_unit) == 0) {
+		factor = cf->external_to_internal;
+	} else {
+		fatal("Expected units of '%s', but got '%s' for '%s'", cf->external_unit, external_unit, field);
 	}
 
-	const char *to_unit = cf->internal_unit;
-	if(to_unit && strcmp("MB", to_unit) == 0) {
-		input_number /= MEGABYTE;
-	}
-
-	*output_number = (int64_t) input_number;
+	*output_number = (int64_t) ceil(input_number * factor);
 
 	return 1;
 }
@@ -147,12 +142,26 @@ double rmsummary_to_external_unit(const char *field, int64_t n) {
 		return (double) n;
 	}
 
-	uintptr_t divider = (uintptr_t) hash_table_lookup(multiplier_of, to_unit);
+	double nd = ((double) n) / cf->external_to_internal;
 
-	if(!divider)
-		fatal("Unknown units: '%s'", to_unit);
+	return nd;
+}
 
-	double nd = ((double) n) / divider;
+double rmsummary_to_base_unit(const char *field, int64_t n) {
+
+	if(!units_initialized)
+		initialize_units();
+
+	struct conversion_field *cf = hash_table_lookup(conversion_fields, field);
+
+	const char *to_unit   = cf->base_unit;
+	const char *from_unit = cf->internal_unit;
+
+	if(from_unit && to_unit && (strcmp(from_unit, to_unit) == 0)) {
+		return (double) n;
+	}
+
+	double nd = ((double) n) * cf->internal_to_base;
 
 	return nd;
 }
@@ -167,35 +176,53 @@ const char *rmsummary_unit_of(const char *key) {
 	return cf->external_unit;
 }
 
+int rmsummary_field_is_float(const char *key) {
+	struct conversion_field *cf = hash_table_lookup(conversion_fields, key);
+
+	if(!cf) {
+		return 0;
+	}
+
+	return cf->float_flag;
+}
+
+
 int rmsummary_assign_char_field(struct rmsummary *s, const char *key, char *value) {
 	if(strcmp(key, "category") == 0) {
-		if(s->category)
-			free(s->category);
+		free(s->category);
 		s->category = xxstrdup(value);
 		return 1;
 	}
 
 	if(strcmp(key, "command") == 0) {
-		if(s->command)
-			free(s->command);
+		free(s->command);
 		s->command = xxstrdup(value);
 		return 1;
 	}
 
 	if(strcmp(key, "exit_type") == 0) {
-		if(s->exit_type)
-			free(s->exit_type);
+		free(s->exit_type);
 		s->exit_type = xxstrdup(value);
 		return 1;
 	}
 
 	if(strcmp(key, "taskid") == 0) {
-		if(s->taskid)
-			free(s->taskid);
+		free(s->taskid);
 		s->taskid = xxstrdup(value);
 		return 1;
 	}
 
+	if(strcmp(key, "task_id") == 0) {
+		free(s->taskid);
+		s->taskid = xxstrdup(value);
+		return 1;
+	}
+
+	if(strcmp(key, "snapshot_name") == 0) {
+		free(s->snapshot_name);
+		s->snapshot_name = xxstrdup(value);
+		return 1;
+	}
 
 	return 0;
 }
@@ -281,10 +308,26 @@ int64_t rmsummary_get_int_field(struct rmsummary *s, const char *key) {
 		return s->cores;
 	}
 
+	if(strcmp(key, "cores_avg") == 0) {
+		return s->cores_avg;
+	}
+
 	if(strcmp(key, "gpus") == 0) {
 		return s->gpus;
 	}
 
+	if(strcmp(key, "machine_cpus") == 0) {
+		return s->machine_cpus;
+	}
+
+	if(strcmp(key, "machine_load") == 0) {
+		return s->machine_load;
+	}
+
+	if(strcmp(key, "snapshots_count") == 0) {
+		return s->snapshots_count;
+	}
+
 	fatal("resource summary does not have a '%s' key. This is most likely a CCTools bug.", key);
 
 
@@ -308,6 +351,10 @@ const char *rmsummary_get_char_field(struct rmsummary *s, const char *key) {
 		return s->taskid;
 	}
 
+	if(strcmp(key, "snapshot_name") == 0) {
+		return s->taskid;
+	}
+
 	fatal("resource summary does not have a '%s' key. This is most likely a CCTools bug.", key);
 
 	return NULL;
@@ -414,16 +461,66 @@ int rmsummary_assign_int_field(struct rmsummary *s, const char *key, int64_t val
 		return 1;
 	}
 
+	if(strcmp(key, "cores_avg") == 0) {
+		s->cores_avg = value;
+		return 1;
+	}
+
+	if(strcmp(key, "machine_cpus") == 0) {
+		s->machine_cpus = value;
+		return 1;
+	}
+
+	if(strcmp(key, "machine_load") == 0) {
+		s->machine_load = value;
+		return 1;
+	}
+
 	if(strcmp(key, "gpus") == 0) {
 		s->gpus = value;
 		return 1;
 	}
 
+	if(strcmp(key, "snapshots_count") == 0) {
+		s->snapshots_count = value;
+		return 1;
+	}
+
 	fatal("resource summary does not have a '%s' key. This is most likely a CCTools bug.", key);
 
 	return 0;
 }
 
+void rmsummary_add_snapshots(struct rmsummary *s, struct jx *array) {
+
+	if(!array) {
+		return;
+	}
+
+	int n = jx_array_length(array);
+
+	if(n < 1) {
+		return;
+	}
+
+	s->snapshots_count = n;
+	s->snapshots       = calloc(n + 1, sizeof(struct rmsummary *));
+	s->snapshots[n]    = NULL; /* terminate in NULL for convenience */
+
+	int count = 0;
+	struct jx *snapshot;
+	for(void *i = NULL; (snapshot = jx_iterate_array(array, &i)); /* nothing */) {
+		struct rmsummary *snap = json_to_rmsummary(snapshot);
+
+		if(!snap) {
+			fatal("malformed resource summary snapshot.");
+		}
+
+		s->snapshots[count] = snap;
+		count++;
+	}
+}
+
 int rmsummary_assign_summary_field(struct rmsummary *s, char *key, struct jx *value) {
 
 	if(strcmp(key, "limits_exceeded") == 0) {
@@ -466,6 +563,8 @@ struct jx *peak_times_to_json(struct rmsummary *s) {
 	peak_time_to_json(output, cf, s, max_concurrent_processes);
 	peak_time_to_json(output, cf, s, cores);
 	peak_time_to_json(output, cf, s, cpu_time);
+	peak_time_to_json(output, cf, s, machine_cpus);
+	peak_time_to_json(output, cf, s, machine_load);
 
 	jx_insert(output, jx_string("units"), jx_string(cf->external_unit));
 
@@ -497,6 +596,8 @@ struct jx *rmsummary_to_json(const struct rmsummary *s, int only_resources) {
 		}
 	}
 
+	field_to_json(output, s, machine_load);
+	field_to_json(output, s, machine_cpus);
 	field_to_json(output, s, disk);
 	field_to_json(output, s, total_files);
 	field_to_json(output, s, bandwidth);
@@ -510,6 +611,7 @@ struct jx *rmsummary_to_json(const struct rmsummary *s, int only_resources) {
 	field_to_json(output, s, total_processes);
 	field_to_json(output, s, max_concurrent_processes);
 	field_to_json(output, s, cores);
+	field_to_json(output, s, cores_avg);
 	field_to_json(output, s, cpu_time);
 	field_to_json(output, s, wall_time);
 	field_to_json(output, s, end);
@@ -550,19 +652,19 @@ struct jx *rmsummary_to_json(const struct rmsummary *s, int only_resources) {
 	return output;
 }
 
-static int json_number_of_array(struct jx *array, char *field, int64_t *number) {
+static int json_number_of_array(struct jx *array, char *field, int64_t *result) {
 
 	struct jx_item *first = array->u.items;
 
 	if(!first)
 		return 0;
 
-	double result;
+	double number;
 
 	if(jx_istype(first->value, JX_DOUBLE)) {
-		result = first->value->u.double_value;
+		number = first->value->u.double_value;
 	} else if(jx_istype(first->value, JX_INTEGER)) {
-		result = (double) first->value->u.integer_value;
+		number = (double) first->value->u.integer_value;
 	} else {
 		return 0;
 	}
@@ -577,7 +679,7 @@ static int json_number_of_array(struct jx *array, char *field, int64_t *number)
 
 	char *unit = second->value->u.string_value;
 
-	return rmsummary_to_internal_unit(field, result, number, unit);
+	return rmsummary_to_internal_unit(field, number, result, unit);
 }
 
 struct rmsummary *json_to_rmsummary(struct jx *j) {
@@ -597,13 +699,18 @@ struct rmsummary *json_to_rmsummary(struct jx *j) {
 		if(jx_istype(value, JX_STRING)) {
 			rmsummary_assign_char_field(s, key, value->u.string_value);
 		} else if(jx_istype(value, JX_INTEGER)) {
-			rmsummary_assign_int_field(s, key, value->u.integer_value);
+			int64_t num;
+			rmsummary_to_internal_unit(key, value->u.integer_value, &num, "external");
+			rmsummary_assign_int_field(s, key, num);
 		} else if(jx_istype(value, JX_ARRAY)) {
 			int64_t number;
 			int status = json_number_of_array(value, key, &number);
 			if(status) {
 				rmsummary_assign_int_field(s, key, number);
 			}
+			if(strcmp(key, "snapshots") == 0) {
+				rmsummary_add_snapshots(s, value);
+			}
 		} else if(jx_istype(value, JX_OBJECT)) {
 			rmsummary_assign_summary_field(s, key, value);
 		}
@@ -611,6 +718,11 @@ struct rmsummary *json_to_rmsummary(struct jx *j) {
 		head = head->next;
 	}
 
+	if(s->wall_time > 0 && s->cpu_time > 0) {
+		//in millicores
+		s->cores_avg = (s->cpu_time * 1000.0)/s->wall_time;
+	}
+
 	return s;
 }
 
@@ -776,6 +888,10 @@ struct rmsummary *rmsummary_create(signed char default_value)
 	s->exit_status = 0;
 	s->signal = 0;
 
+	s->snapshot_name   = NULL;
+	s->snapshots_count = 0;
+	s->snapshots       = NULL;
+
 	return s;
 }
 
@@ -796,12 +912,15 @@ void rmsummary_delete(struct rmsummary *s)
 	if(s->taskid)
 		free(s->taskid);
 
-	if(s->limits_exceeded)
-		rmsummary_delete(s->limits_exceeded);
+	rmsummary_delete(s->limits_exceeded);
+	rmsummary_delete(s->peak_times);
 
-	if(s->peak_times)
-		rmsummary_delete(s->peak_times);
+	int i;
+	for(i = 0; i < s->snapshots_count; i++) {
+		rmsummary_delete(s->snapshots[i]);
+	}
 
+	free(s->snapshots);
 	free(s);
 }
 
@@ -842,9 +961,12 @@ void rmsummary_bin_op(struct rmsummary *dest, const struct rmsummary *src, rm_bi
 	rmsummary_apply_op(dest, src, fn, bandwidth);
 	rmsummary_apply_op(dest, src, fn, total_files);
 	rmsummary_apply_op(dest, src, fn, disk);
-
-	rmsummary_apply_op(dest, src, fn, cores);
 	rmsummary_apply_op(dest, src, fn, fs_nodes);
+	rmsummary_apply_op(dest, src, fn, cores);
+	rmsummary_apply_op(dest, src, fn, cores_avg);
+	rmsummary_apply_op(dest, src, fn, machine_cpus);
+	rmsummary_apply_op(dest, src, fn, machine_load);
+
 }
 
 /* Copy the value for all the fields in src > -1 to dest */
@@ -931,6 +1053,8 @@ static void merge_limits(struct rmsummary *dest, const struct rmsummary *src)
 	merge_limit(dest, src, total_files);
 	merge_limit(dest, src, disk);
 	merge_limit(dest, src, cores);
+	merge_limit(dest, src, cores_avg);
+	merge_limit(dest, src, machine_load);
 	merge_limit(dest, src, fs_nodes);
 
 }
@@ -991,6 +1115,8 @@ void rmsummary_merge_max_w_time(struct rmsummary *dest, const struct rmsummary *
 	max_op_w_time(dest, src, total_files);
 	max_op_w_time(dest, src, disk);
 	max_op_w_time(dest, src, cores);
+	max_op_w_time(dest, src, machine_cpus);
+	max_op_w_time(dest, src, machine_load);
 	max_op_w_time(dest, src, fs_nodes);
 }
 
@@ -1020,6 +1146,24 @@ void rmsummary_merge_min(struct rmsummary *dest, const struct rmsummary *src)
 	}
 }
 
+/* Add summaries together, ignoring negative numbers */
+static int64_t plus(int64_t d, int64_t s)
+{
+	if(d < 0 || s < 0) {
+		return MAX(0, MAX(s, d)); /* return at least 0 */
+	} else {
+		return s + d;
+	}
+}
+
+void rmsummary_add(struct rmsummary *dest, const struct rmsummary *src)
+{
+	if(!dest || !src)
+		return;
+
+	rmsummary_bin_op(dest, src, plus);
+}
+
 void rmsummary_debug_report(const struct rmsummary *s)
 {
 	if(!s)
@@ -1059,6 +1203,10 @@ void rmsummary_debug_report(const struct rmsummary *s)
 		debug(D_DEBUG, "max resource %-18s   : %" PRId64 "\n", "total_files", s->total_files);
 	if(s->disk != -1)
 		debug(D_DEBUG, "max resource %-18s MB: %" PRId64 "\n", "disk", s->disk);
+	if(s->machine_load != -1)
+		debug(D_DEBUG, "max resource %-18s mprocs: %" PRId64 "\n", "machine_load", s->machine_load);
+	if(s->machine_cpus != -1)
+		debug(D_DEBUG, "max resource %-18s cores: %" PRId64 "\n", "machine_cpus", s->machine_cpus);
 }
 
 size_t rmsummary_field_offset(const char *key) {
@@ -1070,6 +1218,10 @@ size_t rmsummary_field_offset(const char *key) {
 		return offsetof(struct rmsummary, cores);
 	}
 
+	if(!strcmp(key, "cores_avg")) {
+		return offsetof(struct rmsummary, cores_avg);
+	}
+
 	if(!strcmp(key, "disk")) {
 		return offsetof(struct rmsummary, disk);
 	}
@@ -1095,35 +1247,43 @@ size_t rmsummary_field_offset(const char *key) {
 	}
 
 	if(!strcmp(key, "bytes_read")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, bytes_read);
 	}
 
 	if(!strcmp(key, "bytes_written")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, bytes_written);
 	}
 
 	if(!strcmp(key, "bytes_received")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, bytes_received);
 	}
 
 	if(!strcmp(key, "bytes_sent")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, bytes_sent);
 	}
 
 	if(!strcmp(key, "bandwidth")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, bandwidth);
 	}
 
 	if(!strcmp(key, "total_files")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, total_files);
 	}
 
 	if(!strcmp(key, "total_processes")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, total_processes);
 	}
 
 	if(!strcmp(key, "max_concurrent_processes")) {
-		return offsetof(struct rmsummary, cpu_time);
+		return offsetof(struct rmsummary, max_concurrent_processes);
+	}
+
+	if(!strcmp(key, "machine_load")) {
+		return offsetof(struct rmsummary, machine_load);
+	}
+
+	if(!strcmp(key, "machine_cpus")) {
+		return offsetof(struct rmsummary, machine_cpus);
 	}
 
 	fatal("Field '%s' was not found.");
@@ -1135,6 +1295,14 @@ int64_t rmsummary_get_int_field_by_offset(const struct rmsummary *s, size_t offs
 	return (*((int64_t *) ((char *) s + offset)));
 }
 
+struct rmsummary *rmsummary_get_snapshot(const struct rmsummary *s, int i) {
+	if(!s || i < 0 || i > s->snapshots_count) {
+		return NULL;
+	}
+
+	return s->snapshots[i];
+}
+
 
 
 /* vim: set noexpandtab tabstop=4: */
