@@ -8,7 +8,7 @@ See the file COPYING for details.
 #ifndef MAKEFLOW_WRAPPER_DOCKER_H
 #define MAKEFLOW_WRAPPER_DOCKER_H
 
-#define CONTAINER_SH "docker.wrapper.sh"
+#define CONTAINER_DOCKER_SH "docker.wrapper.sh"
 
 /*
 This module implements garbage collection on the dag.
@@ -16,12 +16,6 @@ Files that are no longer needed as inputs to any rules
 may be removed, according to a variety of criteria.
 */
 
-typedef enum {
-	CONTAINER_MODE_NONE,
-	CONTAINER_MODE_DOCKER,
-	// CONTAINER_MODE_ROCKET etc
-} container_mode_t;
-
 void makeflow_wrapper_docker_init( struct makeflow_wrapper *w, char *container_image, char *image_tar );
 
 #endif
