@@ -135,6 +135,9 @@ See the file COPYING for details.
 #include "hash_table.h"
 #include "itable.h"
 #include "jx.h"
+#include "jx_print.h"
+#include "jx_pretty_print.h"
+#include "jx_parse.h"
 #include "list.h"
 #include "macros.h"
 #include "path.h"
@@ -145,6 +148,7 @@ See the file COPYING for details.
 
 #include "rmonitor.h"
 #include "rmonitor_poll_internal.h"
+#include "rmonitor_file_watch.h"
 
 #define RESOURCE_MONITOR_USE_INOTIFY 1
 #if defined(RESOURCE_MONITOR_USE_INOTIFY)
@@ -158,11 +162,17 @@ See the file COPYING for details.
 #define DEFAULT_INTERVAL       5               /* in seconds */
 #define DEFAULT_LOG_NAME "resource-pid-%d"     /* %d is used for the value of getpid() */
 
+#define ACTIVATE_DEBUG_FILE ".cctools_resource_monitor_debug"
+
 uint64_t interval = DEFAULT_INTERVAL;
 
 FILE  *log_summary = NULL;      /* Final statistics are written to this file. */
 FILE  *log_series  = NULL;      /* Resource events and samples are written to this file. */
-FILE  *log_inotify  = NULL;      /* List of opened files is written to this file. */
+FILE  *log_inotify = NULL;      /* List of opened files is written to this file. */
+
+char *template_path = NULL;     /* Prefix of all output files names */
+
+int debug_active = 0;           /* 1 if ACTIVATE_DEBUG_FILE exists. If 1, debug info goes to ACTIVATE_DEBUG_FILE ".log" */
 
 struct jx *verbatim_summary_fields; /* fields added to the summary without change */
 
@@ -189,6 +199,8 @@ static char **inotify_watches;  /* Keeps track of created inotify watches. */
 static int alloced_inotify_watches = 0;
 #endif
 
+static int stop_short_running = 0; /* Stop to analyze process that run for less than RESOURCE_MONITOR_SHORT_TIME seconds. By default such processes are not stopped. */
+
 struct itable *wdirs_rc;        /* Counts how many rmonitor_process_info use a rmonitor_wdir_info. */
 struct itable *filesys_rc;      /* Counts how many rmonitor_wdir_info use a rmonitor_filesys_info. */
 
@@ -200,7 +212,8 @@ int lib_helper_extracted;       /* Boolean flag to indicate whether the bundled
                                    helper library was automatically extracted
                                    */
 
-struct rmsummary *summary;
+struct rmsummary *summary;          /* final summary */
+struct rmsummary *snapshot;         /* current snapshot */
 struct rmsummary *resources_limits;
 struct rmsummary *resources_flags;
 
@@ -210,6 +223,17 @@ int64_t total_bytes_tx;   /* total bytes sent */
 
 const char *sh_cmd_line = NULL;    /* command line passed with the --sh option. */
 
+char *snapshot_watch_events_file = NULL;   /* name of the file with a jx document that looks like:
+											  { "FILENAME" : [ { "pattern":"REGEXP", "label":"my snapshot", "max_count":1}, "FILENAME2" : ... ] }
+											  A snapshot is generated when pattern matches a line in the file FILENAME. */
+
+
+struct list *snapshots = NULL;          /* list of snapshots, as json objects. */
+
+struct list *snapshot_labels = NULL;    /* list of labels for current snapshot. */
+
+struct itable *snapshot_watch_pids;     /* pid of all processes watching files for snapshots. */
+
 /***
  * Utility functions (open log files, proc files, measure time)
  ***/
@@ -270,6 +294,34 @@ FILE *open_log_file(const char *log_path)
     return log_file;
 }
 
+void activate_debug_log_if_file() {
+	static timestamp_t last_time = 0;
+
+	timestamp_t current = timestamp_get();
+
+	if((current - last_time) < 30*USECOND ) {
+		return;
+	}
+
+	struct stat s;
+	int status = stat(ACTIVATE_DEBUG_FILE, &s);
+
+	if(status == 0) {
+		if(!debug_active) {
+			debug_active = 1;
+			debug_flags_set("all");
+			debug_config_file(ACTIVATE_DEBUG_FILE ".log");
+			debug_config_file_size(0);
+		}
+	}
+	else if(debug_active) {
+		debug_active = 0;
+		debug_flags_set("clear");
+	}
+
+	last_time = current;
+}
+
 void parse_limit_string(struct rmsummary *limits, char *str)
 {
 	char *pair  = xxstrdup(str);
@@ -345,6 +397,22 @@ void add_verbatim_field(const char *str) {
 	free(pair);
 }
 
+void add_snapshots() {
+	if(!snapshots) {
+		return;
+	}
+
+	struct jx *a = jx_array(0);
+
+	struct jx *j;
+	list_first_item(snapshots);
+	while((j = list_next_item(snapshots))) {
+		jx_array_insert(a, j);
+	}
+
+	jx_insert(verbatim_summary_fields, jx_string("snapshots"), a);
+}
+
 
 int rmonitor_determine_exec_type(const char *executable) {
 	char *absolute_exec = path_which(executable);
@@ -544,7 +612,7 @@ struct rmonitor_wdir_info *lookup_or_create_wd(struct rmonitor_wdir_info *previo
     return inventory;
 }
 
-void rmonitor_add_file_watch(const char *filename, int is_output)
+void rmonitor_add_file_watch(const char *filename, int is_output, int override_flags)
 {
 	struct rmonitor_file_info *finfo;
 	struct stat fst;
@@ -580,7 +648,12 @@ void rmonitor_add_file_watch(const char *filename, int is_output)
 		char **new_inotify_watches;
 		int iwd;
 
-		if ((iwd = inotify_add_watch(rmonitor_inotify_fd, filename, IN_CLOSE_WRITE|IN_CLOSE_NOWRITE|IN_ACCESS|IN_MODIFY)) < 0)
+		int inotify_flags = IN_CLOSE | IN_ACCESS | IN_MODIFY;
+		if(override_flags) {
+			inotify_flags = override_flags;
+		}
+
+		if ((iwd = inotify_add_watch(rmonitor_inotify_fd, filename, inotify_flags)) < 0)
 		{
 			debug(D_RMON, "inotify_add_watch for file %s fails: %s", filename, strerror(errno));
 		} else {
@@ -598,7 +671,7 @@ void rmonitor_add_file_watch(const char *filename, int is_output)
 			}
 			if (iwd < alloced_inotify_watches)
 			{
-				inotify_watches[iwd] = strdup(filename);
+				inotify_watches[iwd] = xxstrdup(filename);
 				if (finfo != NULL) finfo->n_references = 1;
 			} else {
 				debug(D_RMON, "Out of memory: Removing inotify watch for %s", filename);
@@ -609,8 +682,10 @@ void rmonitor_add_file_watch(const char *filename, int is_output)
 #endif
 }
 
-void rmonitor_handle_inotify(void)
+int rmonitor_handle_inotify(void)
 {
+	int urgent = 0;
+
 #if defined(RESOURCE_MONITOR_USE_INOTIFY)
 	struct inotify_event *evdata;
 	struct rmonitor_file_info *finfo;
@@ -622,30 +697,38 @@ void rmonitor_handle_inotify(void)
 	{
 		if (ioctl(rmonitor_inotify_fd, FIONREAD, &nbytes) >= 0)
 		{
-
 			evdata = (struct inotify_event *) malloc(nbytes);
-			if (evdata == NULL) return;
+			if (evdata == NULL) return urgent;
 			if (read(rmonitor_inotify_fd, evdata, nbytes) != nbytes)
 			{
 				free(evdata);
-				return;
+				return urgent;
 			}
 
 			evc = nbytes/sizeof(*evdata);
 			for(i = 0; i < evc; i++)
 			{
-				if (evdata[i].wd >= alloced_inotify_watches) continue;
-				if ((fname = inotify_watches[evdata[i].wd]) == NULL) continue;
+				if (evdata[i].wd >= alloced_inotify_watches) {
+					continue;
+				}
+
+				if ((fname = inotify_watches[evdata[i].wd]) == NULL) {
+					continue;
+				}
+
+				if (finfo == NULL) {
+					continue;
+				}
 				finfo = hash_table_lookup(files, fname);
-				if (finfo == NULL) continue;
 				if (evdata[i].mask & IN_ACCESS) (finfo->n_reads)++;
 				if (evdata[i].mask & IN_MODIFY) (finfo->n_writes)++;
-				if ((evdata[i].mask & IN_CLOSE_WRITE) || (evdata[i].mask & IN_CLOSE_NOWRITE)) {
+				if (evdata[i].mask & IN_CLOSE) {
 					(finfo->n_closes)++;
 					if (stat(fname, &fst) >= 0)
 					{
 						finfo->size_on_close = fst.st_size;
 					}
+
 					/* Decrease reference count and remove watch of zero */
 					(finfo->n_references)--;
 					if (finfo->n_references == 0)
@@ -661,6 +744,8 @@ void rmonitor_handle_inotify(void)
 		free(evdata);
 	}
 #endif
+
+	return urgent;
 }
 
 void append_network_bw(struct rmonitor_msg *msg) {
@@ -750,6 +835,7 @@ void rmonitor_summary_header()
 	    fprintf(log_series, " %s", "bytes_received");
 	    fprintf(log_series, " %s", "bytes_sent");
 	    fprintf(log_series, " %s", "bandwidth");
+	    fprintf(log_series, " %s", "machine_load");
 
 	    if(resources_flags->disk)
 	    {
@@ -769,7 +855,7 @@ struct peak_cores_sample {
 int64_t peak_cores(int64_t wall_time, int64_t cpu_time) {
 	static struct list *samples = NULL;
 
-	int64_t max_separation = 60 + 2*interval; /* at least one minute and a complete measurement round */
+	int64_t max_separation = 60 + 2*interval; /* at least one minute and a complete interval */
 
 	if(!samples) {
 		samples = list_create();
@@ -787,9 +873,9 @@ int64_t peak_cores(int64_t wall_time, int64_t cpu_time) {
 
 	struct peak_cores_sample *head;
 
-	/* Drop entries older than max_separation, unless we only have three samples. */
+	/* Drop entries older than max_separation, unless we only have two samples. */
 	while((head = list_peek_head(samples))) {
-		if(list_size(samples) < 3) {
+		if(list_size(samples) < 2) {
 			break;
 		}
 		else if( head->wall_time + max_separation*USECOND < tail->wall_time) {
@@ -805,14 +891,24 @@ int64_t peak_cores(int64_t wall_time, int64_t cpu_time) {
 	int64_t diff_wall = tail->wall_time - head->wall_time;
 	int64_t diff_cpu  = tail->cpu_time  - head->cpu_time;
 
-	/* hack to elimiate noise. if diff_wall < 60s, we return 1. If command runs
-	 * for more than 60s, the average cpu/wall serves as a fallback in the
-	 * final summary. */
 
-	if(diff_wall < 60 * USECOND) {
+	if(diff_wall < 60) {
+		/* hack to elimiate noise. if diff_wall < 60s, we return 1. If command runs
+		 * for more than 60s, the average cpu/wall serves as a fallback in the
+		 * final summary. */
 		return 1;
 	} else {
-		return (int64_t) MAX(1, ceil( ((double) diff_cpu)/diff_wall));
+		/* hack to eliminate noise. we do not round up unless more than %10 of
+		 * the additional core is used. */
+
+		double raw   = MAX(1, ((double) diff_cpu)/diff_wall);
+		double floor = trunc(raw);
+
+		if(raw - floor > 0.1) {
+			return (int64_t) ceil(raw);
+		} else {
+			return (int64_t) floor;
+		}
 	}
 }
 
@@ -823,6 +919,11 @@ void rmonitor_collate_tree(struct rmsummary *tr, struct rmonitor_process_info *p
 
 	tr->cores      = peak_cores(tr->wall_time, tr->cpu_time);
 
+	tr->cores_avg  = 0;
+	if(tr->wall_time > 0) {
+		tr->cores_avg = (tr->cpu_time * 1000) / tr->wall_time;
+	}
+
 	tr->max_concurrent_processes = (int64_t) itable_size(processes);
 	tr->total_processes          = summary->total_processes;
 
@@ -853,6 +954,9 @@ void rmonitor_collate_tree(struct rmsummary *tr, struct rmonitor_process_info *p
 	tr->disk = (int64_t) (d->byte_count + ONE_MEGABYTE - 1) / ONE_MEGABYTE;
 
 	tr->fs_nodes          = (int64_t) f->disk.f_ffree;
+
+	tr->machine_load = p->load.last_minute;
+	tr->machine_cpus = p->load.cpus;
 }
 
 void rmonitor_find_max_tree(struct rmsummary *result, struct rmsummary *tr)
@@ -884,6 +988,7 @@ void rmonitor_log_row(struct rmsummary *tr)
 		fprintf(log_series, " %" PRId64, tr->bytes_received);
 		fprintf(log_series, " %" PRId64, tr->bytes_sent);
 		fprintf(log_series, " %" PRId64, tr->bandwidth);
+		fprintf(log_series, " %04.2lf" PRId64, tr->machine_load / 1000.0);
 
 		if(resources_flags->disk)
 		{
@@ -904,6 +1009,57 @@ void rmonitor_log_row(struct rmsummary *tr)
 
 }
 
+int record_snapshot(struct rmsummary *tr) {
+	if(list_size(snapshot_labels) < 1) {
+		return 0;
+	}
+
+	buffer_t b;
+	buffer_init(&b);
+
+	const char *sep ="";
+
+	char *str;
+	while((str = list_pop_head(snapshot_labels))) {
+		string_chomp(str);
+		buffer_printf(&b, "%s%s", sep, str);
+		sep = ",";
+		free(str);
+	}
+
+	if(!snapshots) {
+		snapshots = list_create();
+	}
+
+	snapshot->end       = usecs_since_epoch();
+	snapshot->wall_time = snapshot->end - snapshot->start;
+
+	struct jx *j = rmsummary_to_json(tr, /* only resources */ 1);
+
+	jx_insert_string(j, "snapshot_name", buffer_tostring(&b));
+
+	char *output_file = string_format("%s.snapshot.%02d", template_path, list_size(snapshots));
+	FILE *snap_f = fopen(output_file, "w");
+
+	if(!snap_f) {
+		warn(D_WQ, "Could not save snapshots file '%s': %s", output_file, strerror(errno));
+	} else {
+		jx_pretty_print_stream(j, snap_f);
+		fclose(snap_f);
+	}
+
+	free(output_file);
+
+	/* push to the front, since snapshots are writen in reverse order. */
+	list_push_head(snapshots, j);
+
+	debug(D_RMON, "Recoded snapshot: '%s'", buffer_tostring(&b));
+
+	buffer_free(&b);
+
+	return 1;
+}
+
 void decode_zombie_status(struct rmsummary *summary, int wait_status)
 {
 	/* update from any END_WAIT message received. */
@@ -1027,14 +1183,17 @@ int rmonitor_final_summary()
 	add_verbatim_field(monitor_self_info);
 
 	char hostname[DOMAIN_NAME_MAX];
-	domain_name_cache_guess(hostname);
 
 	char *host_info = NULL;
-	if(strlen(hostname) > 0) {
+	if(domain_name_cache_guess(hostname)) {
 		host_info = string_format("host:%s", hostname);
 		add_verbatim_field(host_info);
 	}
 
+	if(snapshots && list_size(snapshots) > 0) {
+		add_snapshots();
+	}
+
 	if(log_inotify)
 	{
 		rmonitor_find_files_final_sizes();
@@ -1057,7 +1216,9 @@ int rmonitor_final_summary()
 			count = select(nfds, &rset, NULL, NULL, &timeout);
 
 			if(count > 0)
-				if (FD_ISSET(rmonitor_inotify_fd, &rset)) rmonitor_handle_inotify();
+				if (FD_ISSET(rmonitor_inotify_fd, &rset)) {
+					rmonitor_handle_inotify();
+				}
 		} while(count > 0);
 
 		rmonitor_file_io_summaries();
@@ -1099,13 +1260,19 @@ void rmonitor_track_process(pid_t pid)
 	char *newpath;
 	struct rmonitor_process_info *p;
 
-	if(!ping_process(pid))
+	if(!pid) {
 		return;
+	}
+
+	if(!ping_process(pid)) {
+		return;
+	}
 
 	p = itable_lookup(processes, pid);
 
-	if(p)
+	if(p) {
 		return;
+	}
 
 	p = malloc(sizeof(struct rmonitor_process_info));
 	bzero(p, sizeof(struct rmonitor_process_info));
@@ -1124,6 +1291,7 @@ void rmonitor_track_process(pid_t pid)
 	p->running = 1;
 	p->waiting = 0;
 
+	rmonitor_poll_process_once(p);
 	summary->total_processes++;
 }
 
@@ -1188,6 +1356,49 @@ void ping_processes(void)
         }
 }
 
+void set_snapshot_watch_events() {
+
+	if(!snapshot_watch_events_file) {
+		return;
+	}
+
+	struct jx *j = jx_parse_file(snapshot_watch_events_file);
+
+	if(!j) {
+		debug(D_FATAL, "Could not process '%s' snapshots file.", snapshot_watch_events_file);
+		exit(RM_MONITOR_ERROR);
+	}
+
+	void *i = NULL;
+	struct jx *key;
+	while((key = jx_iterate_keys(j, &i))) {
+		const char *fname = key->u.string_value;
+		struct jx *array  = jx_lookup(j, fname);
+
+		if(!jx_istype(array , JX_OBJECT)) {
+			debug(D_FATAL, "Error processing snapshot configurations for %s. Not of the form {\"FILENAME\" : { \"events\" : [ { \"label\": ..., }, ... ]", fname);
+			exit(RM_MONITOR_ERROR);
+		}
+
+		pid_t pid = rmonitor_watch_file(fname, array);
+
+		itable_insert(snapshot_watch_pids, (uintptr_t) pid, (void *) snapshot_watch_pids);
+	}
+
+	jx_delete(j);
+}
+
+void terminate_snapshot_watch_events() {
+  uint64_t pid;
+  void *dummy;
+
+  itable_firstkey(snapshot_watch_pids);
+  while(itable_nextkey(snapshot_watch_pids, &pid, &dummy)) {
+        kill(pid, SIGKILL);
+  }
+
+}
+
 struct rmsummary *rmonitor_final_usage_tree(void)
 {
     struct rusage usg;
@@ -1348,6 +1559,8 @@ void rmonitor_final_cleanup(int signum)
     if(log_inotify)
 	    fclose(log_inotify);
 
+	terminate_snapshot_watch_events();
+
     exit(status);
 }
 
@@ -1432,7 +1645,19 @@ int rmonitor_dispatch_msg(void)
 	struct rmonitor_msg msg;
 	struct rmonitor_process_info *p;
 
-	recv_monitor_msg(rmonitor_queue_fd, &msg);
+	int recv_status = recv_monitor_msg(rmonitor_queue_fd, &msg);
+
+	if(recv_status < 0) {
+		if(errno != EAGAIN) {
+			debug(D_RMON, "Error receiving message: %s", strerror(errno));
+			return 1;
+		}
+	}
+
+	if(((unsigned int) recv_status) < sizeof(msg)) {
+		debug(D_RMON, "Malformed message from monitored processes. Ignoring.");
+		return 1;
+	}
 
 	//Next line commented: Useful for detailed debugging, but too spammy for regular operations.
 	//debug(D_RMON,"message '%s' (%d) from %d with status '%s' (%d)\n", str_msgtype(msg.type), msg.type, msg.origin, strerror(msg.error), msg.error);
@@ -1442,15 +1667,16 @@ int rmonitor_dispatch_msg(void)
 	if(!p)
 	{
 		/* We either got a malformed message, message from a
-		process we are not tracking anymore, or a message from
-		a newly created process.  */
+		process we are not tracking anymore, a message from
+		a newly created process, or a message from a snapshot process.  */
 		if( msg.type == END_WAIT )
         {
 			release_waiting_process(msg.origin);
 			return 1;
         }
-		else if(msg.type != BRANCH)
+		else if(msg.type != BRANCH && msg.type != SNAPSHOT) {
 			return 1;
+		}
 	}
 
     switch(msg.type)
@@ -1458,14 +1684,16 @@ int rmonitor_dispatch_msg(void)
         case BRANCH:
 			msg.error = 0;
             rmonitor_track_process(msg.origin);
-            if(summary->max_concurrent_processes < itable_size(processes))
+            if(summary->max_concurrent_processes < itable_size(processes)) {
                 summary->max_concurrent_processes = itable_size(processes);
+			}
             break;
         case END_WAIT:
 			msg.error = 0;
             p->waiting = 1;
-			if(msg.origin == first_process_pid)
+			if(msg.origin == first_process_pid) {
 				first_process_exit_status = msg.data.n;
+			}
             break;
         case END:
 			msg.error = 0;
@@ -1473,15 +1701,18 @@ int rmonitor_dispatch_msg(void)
             break;
         case CHDIR:
 			msg.error = 0;
-			if(follow_chdir)
+			if(follow_chdir) {
 				p->wd = lookup_or_create_wd(p->wd, msg.data.s);
+			}
             break;
 		case OPEN_INPUT:
 		case OPEN_OUTPUT:
 			switch(msg.error) {
 				case 0:
 					debug(D_RMON, "File %s has been opened.\n", msg.data.s);
-					rmonitor_add_file_watch(msg.data.s, msg.type == OPEN_OUTPUT);
+					if(log_inotify) {
+						rmonitor_add_file_watch(msg.data.s, msg.type == OPEN_OUTPUT, 0);
+					}
 					break;
 				case EMFILE:
 					/* Eventually report that we ran out of file descriptors. */
@@ -1522,6 +1753,10 @@ int rmonitor_dispatch_msg(void)
 					break;
 			}
             break;
+		case SNAPSHOT:
+			debug(D_RMON, "Snapshot msg label: '%s'\n", msg.data.s);
+			list_push_tail(snapshot_labels, xxstrdup(msg.data.s));
+			break;
         default:
             break;
     };
@@ -1531,11 +1766,34 @@ int rmonitor_dispatch_msg(void)
 	if(!rmonitor_check_limits(summary))
 		rmonitor_final_cleanup(SIGTERM);
 
-	if(msg.type == BRANCH || msg.type == END_WAIT || msg.type == END) {
+	// find out if messages are urgent:
+	if(msg.type == SNAPSHOT) {
+		// SNAPSHOTs are always urgent
 		return 1;
-	} else {
-		return 0;
 	}
+
+	if(msg.type == END_WAIT || msg.type == END) {
+		if(msg.origin == first_process_pid) {
+			// ENDs from the first process are always urgent.
+			return 1;
+		}
+
+		if(stop_short_running) {
+			// we are stopping all processes, so all ENDs are urgent.
+			return 1;
+		}
+
+		if(msg.end < (msg.start + RESOURCE_MONITOR_SHORT_TIME)) {
+			// for short running processes END_WAIT and END are not urgent.
+			return 0;
+		}
+
+		// ENDs for long running processes are always urgent.
+		return 1;
+	}
+
+	// Any other case is not urgent.
+	return 0;
 }
 
 int wait_for_messages(int interval)
@@ -1557,25 +1815,35 @@ int wait_for_messages(int interval)
 	{
 
 		/* Figure out the number of file descriptors to pass to select */
-		int nfds = rmonitor_queue_fd + 1;
+		int nfds = 1 + MAX(rmonitor_queue_fd, rmonitor_inotify_fd);
 		fd_set rset;
 
-		int count = 0;
+		int urgent = 0;
+		int count  = 0;
 		do
 		{
 			FD_ZERO(&rset);
-			if (rmonitor_queue_fd > 0)   FD_SET(rmonitor_queue_fd,   &rset);
+			if (rmonitor_queue_fd > 0) {
+				FD_SET(rmonitor_queue_fd,   &rset);
+			}
+
+			if (rmonitor_inotify_fd > 0) {
+				FD_SET(rmonitor_inotify_fd, &rset);
+			}
 
 			count = select(nfds, &rset, NULL, NULL, &timeout);
 
-			if(count > 0) {
-				if (FD_ISSET(rmonitor_queue_fd, &rset)) {
-					int urgent = rmonitor_dispatch_msg();
-					if(urgent) {
-						timeout.tv_sec  = 0;
-						timeout.tv_usec = 0;
-					}
-				}
+			if (FD_ISSET(rmonitor_queue_fd, &rset)) {
+				urgent |= rmonitor_dispatch_msg();
+			}
+
+			if (FD_ISSET(rmonitor_inotify_fd, &rset)) {
+				urgent |= rmonitor_handle_inotify();
+			}
+
+			if(urgent) {
+				timeout.tv_sec  = 0;
+				timeout.tv_usec = 0;
 			}
 		} while(count > 0);
 	}
@@ -1648,6 +1916,7 @@ struct rmonitor_process_info *spawn_first_process(const char *executable, char *
 
         if (child_in_foreground)
         {
+			signal(SIGTTOU, SIG_IGN);
             int fdtty, retc;
             fdtty = open("/dev/tty", O_RDWR);
             if (fdtty >= 0)
@@ -1668,7 +1937,7 @@ struct rmonitor_process_info *spawn_first_process(const char *executable, char *
 
 		char *executable_path = path_which(executable);
 		if(executable_path) {
-			rmonitor_add_file_watch(executable_path, /* is output? */ 0);
+			rmonitor_add_file_watch(executable_path, /* is output? */ 0, 0);
 			free(executable_path);
 		}
     }
@@ -1678,6 +1947,8 @@ struct rmonitor_process_info *spawn_first_process(const char *executable, char *
 	}
     else //child
     {
+        setpgid(0, 0);
+
         debug(D_RMON, "executing: %s\n", executable);
 
 		errno = 0;
@@ -1702,7 +1973,8 @@ static void show_help(const char *cmd)
     fprintf(stdout, "%-30s Show this message.\n", "-h,--help");
     fprintf(stdout, "%-30s Show version string.\n", "-v,--version");
     fprintf(stdout, "\n");
-    fprintf(stdout, "%-30s Interval between observations, in seconds. (default=%d)\n", "-i,--interval=<n>", DEFAULT_INTERVAL);
+    fprintf(stdout, "%-30s Maximum interval between observations, in seconds. (default=%d)\n", "-i,--interval=<n>", DEFAULT_INTERVAL);
+    fprintf(stdout, "%-30s Accurately measure short running processes (adds overhead).\n", "--accurate-short-processes");
     fprintf(stdout, "%-30s Read command line from <str>, and execute as '/bin/sh -c <str>'\n", "-c,--sh=<str>");
     fprintf(stdout, "\n");
     fprintf(stdout, "%-30s Use maxfile with list of var: value pairs for resource limits.\n", "-l,--limits-file=<maxfile>");
@@ -1711,18 +1983,21 @@ static void show_help(const char *cmd)
     fprintf(stdout, "\n");
     fprintf(stdout, "%-30s Keep the monitored process in foreground (for interactive use).\n", "-f,--child-in-foreground");
     fprintf(stdout, "\n");
-    fprintf(stdout, "%-30s Follow the size of processes' current working directories. \n", "--follow-chdir");
-    fprintf(stdout, "%-30s Follow the size of <dir>. If not specified, follow the current directory.\n", "--measure-dir");
-    fprintf(stdout, "%-30s Can be specified multiple times.\n", "");
-    fprintf(stdout, "\n");
     fprintf(stdout, "%-30s Specify filename template for log files (default=resource-pid-<pid>)\n", "-O,--with-output-files=<file>");
     fprintf(stdout, "%-30s Write resource time series to <template>.series\n", "--with-time-series");
     fprintf(stdout, "%-30s Write inotify statistics of opened files to default=<template>.files\n", "--with-inotify");
     fprintf(stdout, "%-30s Include this string verbatim in a line in the summary. \n", "-V,--verbatim-to-summary=<str>");
     fprintf(stdout, "%-30s (Could be specified multiple times.)\n", "");
     fprintf(stdout, "\n");
-    fprintf(stdout, "%-30s Do not measure working directory footprint.\n", "--without-disk-footprint");
+    fprintf(stdout, "%-30s Follow the size of <dir>. By default the directory at the start of\n", "--measure-dir=<dir>");
+    fprintf(stdout, "%-30s execution is followed. Can be specified multiple times.\n", "");
+    fprintf(stdout, "%-30s See --without-disk-footprint below.\n", "");
+    fprintf(stdout, "%-30s Do not measure working directory footprint. Overrides --measure-dir and --follow-chdir.\n", "--without-disk-footprint");
+    fprintf(stdout, "\n");
+    fprintf(stdout, "\n");
     fprintf(stdout, "%-30s Do not pretty-print summaries.\n", "--no-pprint");
+    fprintf(stdout, "\n");
+    fprintf(stdout, "%-30s Configuration file for snapshots on file patterns. (See man page.)\n", "--snapshot-events=<file>");
 }
 
 
@@ -1746,6 +2021,7 @@ int rmonitor_resources(long int interval /*in seconds */)
 	while(itable_size(processes) > 0)
 	{
 		debug(D_RMON, "Round %" PRId64, round);
+		activate_debug_log_if_file();
 
 		resources_now->last_error = 0;
 
@@ -1760,7 +2036,8 @@ int rmonitor_resources(long int interval /*in seconds */)
 		// rmonitor_fss_once(f); disabled until statfs fs id makes sense.
 
 		rmonitor_collate_tree(resources_now, p_acc, m_acc, d_acc, f_acc);
-		rmonitor_find_max_tree(summary, resources_now);
+		rmonitor_find_max_tree(summary,  resources_now);
+		rmonitor_find_max_tree(snapshot, resources_now);
 		rmonitor_log_row(resources_now);
 
 		if(!rmonitor_check_limits(summary))
@@ -1769,6 +2046,13 @@ int rmonitor_resources(long int interval /*in seconds */)
 		release_waiting_processes();
 
 		cleanup_zombies();
+
+		if(record_snapshot(snapshot)) {
+			rmsummary_delete(snapshot);
+			snapshot = calloc(1, sizeof(*snapshot));
+			snapshot->start = usecs_since_epoch();
+		}
+
 		//If no more process are alive, break out of loop.
 		if(itable_size(processes) < 1)
 			break;
@@ -1797,7 +2081,6 @@ int main(int argc, char **argv) {
     char *executable;
     int64_t c;
 
-    char *template_path = NULL;
     char *summary_path = NULL;
     char *series_path  = NULL;
     char *opened_path  = NULL;
@@ -1815,7 +2098,9 @@ int main(int argc, char **argv) {
     signal(SIGQUIT, rmonitor_final_cleanup);
     signal(SIGTERM, rmonitor_final_cleanup);
 
-    summary          = calloc(1, sizeof(struct rmsummary));
+    summary  = calloc(1, sizeof(struct rmsummary));
+    snapshot = calloc(1, sizeof(struct rmsummary));
+
     summary->peak_times = rmsummary_create(-1);
     resources_limits = rmsummary_create(-1);
     resources_flags  = rmsummary_create(0);
@@ -1824,6 +2109,9 @@ int main(int argc, char **argv) {
 	total_bytes_tx = 0;
 	tx_rx_sizes    = list_create();
 
+	snapshot_labels = list_create();
+	snapshot_watch_pids = itable_create(0);
+
     rmsummary_read_env_vars(resources_limits);
 
     processes = itable_create(0);
@@ -1845,7 +2133,10 @@ int main(int argc, char **argv) {
 		LONG_OPT_WORKING_DIRECTORY,
 		LONG_OPT_FOLLOW_CHDIR,
 		LONG_OPT_MEASURE_DIR,
-		LONG_OPT_NO_PPRINT
+		LONG_OPT_NO_PPRINT,
+		LONG_OPT_SNAPSHOT_FILE,
+		LONG_OPT_SNAPSHOT_WATCH_CONF,
+		LONG_OPT_STOP_SHORT_RUNNING
 	};
 
     static const struct option long_options[] =
@@ -1866,11 +2157,16 @@ int main(int argc, char **argv) {
 		    {"measure-dir",  required_argument, 0,  LONG_OPT_MEASURE_DIR},
 		    {"no-pprint",    no_argument,       0,  LONG_OPT_NO_PPRINT},
 
+		    {"accurate-short-processes", no_argument, 0, LONG_OPT_STOP_SHORT_RUNNING},
+
 		    {"with-output-files",      required_argument, 0,  'O'},
 		    {"with-time-series",       no_argument, 0, LONG_OPT_TIME_SERIES},
 		    {"with-inotify",           no_argument, 0, LONG_OPT_OPENED_FILES},
 		    {"without-disk-footprint", no_argument, 0, LONG_OPT_NO_DISK_FOOTPRINT},
 
+		    {"snapshot-file",   required_argument, 0, LONG_OPT_SNAPSHOT_FILE},
+		    {"snapshot-events", required_argument, 0, LONG_OPT_SNAPSHOT_WATCH_CONF},
+
 		    {0, 0, 0, 0}
 	    };
 
@@ -1933,6 +2229,7 @@ int main(int argc, char **argv) {
 				break;
 			case LONG_OPT_NO_DISK_FOOTPRINT:
 				resources_flags->disk = 0;
+				follow_chdir = 0;
 				break;
 			case LONG_OPT_FOLLOW_CHDIR:
 				follow_chdir = 1;
@@ -1944,9 +2241,18 @@ int main(int argc, char **argv) {
 					exit(RM_MONITOR_ERROR);
 				}
 				break;
+			case LONG_OPT_STOP_SHORT_RUNNING:
+				stop_short_running = 1;
+				break;
 			case LONG_OPT_NO_PPRINT:
 				pprint_summaries = 0;
 				break;
+			case LONG_OPT_SNAPSHOT_FILE:
+				fatal("This option has been replaced with --snapshot-events. Please consult the manual of resource_monitor.");
+				break;
+			case LONG_OPT_SNAPSHOT_WATCH_CONF:
+				snapshot_watch_events_file = xxstrdup(optarg);
+				break;
 			default:
 				show_help(argv[0]);
 				return 1;
@@ -2009,10 +2315,8 @@ int main(int argc, char **argv) {
         fatal("error executing %s: %s\n", command_line, strerror(errno));
     }
 
-#ifdef CCTOOLS_USE_RMONITOR_HELPER_LIB
     write_helper_lib();
-    rmonitor_helper_init(lib_helper_name, &rmonitor_queue_fd);
-#endif
+    rmonitor_helper_init(lib_helper_name, &rmonitor_queue_fd, stop_short_running);
 
 	summary_path = default_summary_name(template_path);
 
@@ -2028,16 +2332,15 @@ int main(int argc, char **argv) {
 
     summary->command = xxstrdup(command_line);
     summary->start   = usecs_since_epoch();
-
+    snapshot->start  = summary->start;
 
 #if defined(RESOURCE_MONITOR_USE_INOTIFY)
     if(log_inotify)
     {
-	    rmonitor_inotify_fd = inotify_init();
+	    rmonitor_inotify_fd     = inotify_init();
 	    alloced_inotify_watches = 100;
-	    inotify_watches = (char **)(calloc(alloced_inotify_watches, sizeof(char *)));
-	    if (inotify_watches == NULL) alloced_inotify_watches = 0;
-    }
+	    inotify_watches         = (char **) calloc(alloced_inotify_watches, sizeof(char *));
+	}
 #endif
 
 	/* if we are not following changes in directory, and no directory was manually added, we follow the current working directory. */
@@ -2052,6 +2355,8 @@ int main(int argc, char **argv) {
 		exit(RM_MONITOR_ERROR);
 	}
 
+	set_snapshot_watch_events();
+
 	spawn_first_process(executable, argv + optind, child_in_foreground);
     rmonitor_resources(interval);
     rmonitor_final_cleanup(SIGTERM);
