@@ -149,6 +149,8 @@ struct work_queue_task {
 	struct rmsummary *resources_requested;                 /**< Number of cores, disk, memory, time, etc. the task requires. */
 	char *monitor_output_directory;                        /**< Custom output directory for the monitoring output files. If NULL, save to directory from @ref work_queue_enable_monitoring */
 
+	char *monitor_snapshot_file;                          /**< Filename the monitor checks to produce snapshots. */
+
 	/* deprecated fields */
 	//int total_submissions;                                 /**< @deprecated Use try_count. */
 
@@ -242,6 +244,9 @@ struct work_queue_stats {
 	int capacity_cores;     /**< The estimated number of workers' cores that this master can effectively support.*/
 	int capacity_memory;    /**< The estimated number of workers' MB of RAM that this master can effectively support.*/
 	int capacity_disk;      /**< The estimated number of workers' MB of disk that this master can effectively support.*/
+	int capacity_instantaneous; /**< The estimated number of tasks that this master can support considering only the most recently completed task. */
+	int capacity_weighted; /**< The estimated number of tasks that this master can support placing greater weight on the most recently completed task. */
+	double capacity_weight;			/**< The weight value placed on the capacity_weighted calculation. */
 
 	int64_t total_cores;      /**< Total number of cores aggregated across the connected workers. */
 	int64_t total_memory;     /**< Total memory in MB aggregated across the connected workers. */
@@ -497,6 +502,60 @@ This may be called on tasks after they are returned from @ref work_queue_wait.
 */
 void work_queue_task_delete(struct work_queue_task *t);
 
+
+/** When monitoring, indicates a json-encoded file that instructs the
+monitor to take a snapshot of the task resources. Snapshots appear in the JSON
+summary file of the task, under the key "snapshots". Snapshots are taken on
+events on files described in the monitor_snapshot_file. The monitor_snapshot_file
+is a json encoded file with the following format:
+
+    {
+        "FILENAME": {
+            "from-start":boolean,
+            "from-start-if-truncated":boolean,
+            "delete-if-found":boolean,
+            "events": [
+                {
+                    "label":"EVENT_NAME",
+                    "on-create":boolean,
+                    "on-truncate":boolean,
+                    "pattern":"REGEXP",
+                    "count":integer
+                },
+                {
+                    "label":"EVENT_NAME",
+                    ...
+                }
+            ]
+        },
+        "FILENAME": {
+            ...
+    }
+
+All fields but label are optional.
+
+            from-start:boolean         If FILENAME exits when task starts running, process from line 1. Default: false, as the task may be appending to an already existing file.
+            from-start-if-truncated    If FILENAME is truncated, process from line 1. Default: true, to account for log rotations.
+            delete-if-found            Delete FILENAME when found. Default: false
+
+            events:
+            label        Name that identifies the snapshot. Only alphanumeric, -,
+                         and _ characters are allowed. 
+            on-create    Take a snapshot every time the file is created. Default: false
+            on-truncate  Take a snapshot when the file is truncated.    Default: false
+            pattern      Take a snapshot when a line matches the regexp pattern.    Default: none
+            count        Maximum number of snapshots for this label. Default: -1 (no limit)
+
+For more information, consult the manual of the resource_monitor.
+
+@param t A work queue task object.
+@param monitor_snapshot_file A filename.
+@return 1 if the task file is successfully specified, 0 if either of @a t, or @a monitor_snapshot_file is null.
+*/
+
+int work_queue_specify_snapshot_file(struct work_queue_task *t, const char *monitor_snapshot_file);
+
+
 //@}
 
 /** @name Functions - Queues */
@@ -867,9 +926,9 @@ void work_queue_specify_keepalive_interval(struct work_queue *q, int interval);
 void work_queue_specify_keepalive_timeout(struct work_queue *q, int timeout);
 
 /** Set the preference for using hostname over IP address to connect.
-IP uses IP address (standard behavior), HOSTNAME uses hostname provided by master
+'by_ip' uses IP address (standard behavior), or 'by_hostname' to use the hostname at the master.
 @param q A work queue object.
-@param preferred_connection An string to indicate using IP or HOSTNAME.
+@param preferred_connection An string to indicate using 'by_ip' or a 'by_hostname'.
 */
 void work_queue_master_preferred_connection(struct work_queue *q, const char *preferred_connection);
 
